function createResource(initialConfig) {
    if (typeof initialConfig === "string") {
        initialConfig = {
            path: initialConfig,
        };
    }
    var path = initialConfig.path;
    return function (transport) {
        return function () {
            var _a = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                _a[_i] = arguments[_i];
            }
            var params = _a[0];
            var publicConfig = Object.assign({}, initialConfig, {
                path: typeof path === "function" ? path(params === null || params === void 0 ? void 0 : params.query) : path,
            });
            if (params && typeof params.payload !== "undefined") {
                publicConfig.payload = params.payload;
            }
            return transport(publicConfig);
        };
    };
}

var DEFAULTS = {
    method: "GET",
    middlewares: [],
    headers: {},
    baseUrl: "/",
};

var TRANSPORT_DEFAULTS = {
    baseUrl: DEFAULTS.baseUrl,
    middlewares: DEFAULTS.middlewares,
    headers: DEFAULTS.headers,
};
function createTransport(initialConfig) {
    var config = Object.assign({}, TRANSPORT_DEFAULTS, initialConfig);
    var transport = function (resourceConfig) {
        var _a;
        return config.fetcher({
            url: buildUrl(config.baseUrl, resourceConfig.path),
            method: (_a = resourceConfig.method) !== null && _a !== void 0 ? _a : DEFAULTS.method,
            headers: Object.assign({}, config.headers, resourceConfig.headers),
            payload: resourceConfig.payload,
        });
    };
    return applyTransportMiddlewares(transport, config.middlewares);
}
function applyTransportMiddlewares(transport, middlewares) {
    if (!Array.isArray(middlewares)) {
        middlewares = [middlewares];
    }
    for (var _i = 0, middlewares_1 = middlewares; _i < middlewares_1.length; _i++) {
        var middleware = middlewares_1[_i];
        transport = middleware(transport);
    }
    return transport;
}
function buildUrl(baseUrl, path) {
    return "".concat(baseUrl).concat(path);
}

function createClient(transport, resources) {
    return Object.keys(resources).reduce(function (accumulator, current) {
        accumulator[current] = resources[current](transport);
        return accumulator;
    }, {});
}

function applyMiddleware(resource, middlewares) {
    return function (transport) {
        return resource(applyTransportMiddlewares(transport, middlewares));
    };
}

export { applyMiddleware, createClient, createResource, createTransport };
